
/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1
Created on Date : 02/02/2013
Last update     : 15/12/2014

CopyRight 2006-2014 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library supports the Real Time Clock Calendar DS1337 or compatible
devices. The library is kept compatible with the PCF8563

The following functions have been added:

 set_seconds_alarm_RTCC (seconds associated to the first alarm)

 The second alarm is not handled but it can be easily implemented.

*******************************************************************************/


#ifndef DS1337_H
#define DS1337_H

#ifdef __XC8
    #include <xc.h>
        #ifndef _PIC18
        #error The PCF1337 Library supports only PIC18 devices
    #endif
#endif

#include <i2c.h>


//**************************************************
// you must change this address if
// you change the device.
//**************************************************

#define RTCC_WRITE_ADD 0xD0

//**************************************************
// Definition of Day constants (Italian, English)
//**************************************************

// Italian
#define	RTCC_DO 0x00
#define	RTCC_LU 0x01
#define	RTCC_MA 0x02
#define	RTCC_ME 0x03
#define	RTCC_GI 0x04
#define	RTCC_VE 0x05
#define	RTCC_SA 0x06

// English
#define	RTCC_SU 0x00
#define	RTCC_MO 0x01
#define	RTCC_TU 0x02
#define	RTCC_WE 0x03
#define	RTCC_TR 0x04
#define	RTCC_FR 0x05
#define	RTCC_SA 0x06



//**************************************************
//               Standard Constants
// enable value for the alarm bit
//**************************************************

#define RTCC_ENABLE_ON  0b00000000
#define RTCC_ENABLE_OFF 0b10000000


//**************************************************
//           Address constants
//**************************************************

#define RTCC_SECONDS_ADDR 		0x00
#define RTCC_MINUTES_ADDR 		0x01
#define RTCC_HOURS_ADDR 		0x02
#define RTCC_DAYS_ADDR 			0x04
#define RTCC_MONTHS_ADDR		0x05
#define RTCC_YEARS_ADDR	 		0x06
#define RTCC_SECONDS_ALARM_ADDR	 	0x07
#define RTCC_MINUTS_ALARM_ADDR	 	0x08
#define RTCC_HOURS_ALARM_ADDR         	0x09
#define RTCC_DAYS_ALARM_ADDR	 	0x0A
#define RTCC_MINUTS_ALARM_2_ADDR 	0x0B
#define RTCC_HOURS_ALARM_2_ADDR	 	0x0C
#define RTCC_DAYS_ALARM_2_ADDR	 	0x0D
#define RTCC_CONTROL_REG_1_ADDR		0x0E
#define RTCC_CONTROL_REG_2_ADDR		0x0F

//**************************************************
//           Limits constants
//**************************************************
#define RTCC_MAX_MINUTES 0x60
#define RTCC_MAX_HOURS 	 0x24
#define RTCC_MAX_YEARS 	 0xA0
#define RTCC_MAX_MONTHS	 0x13
#define RTCC_MAX_DAYS 	 0x32

/**
 * This function initialize the I2C module accordingly to the clock and baud rate.
 *
 *
 * @param crystal_frequency_MHz Clock frequency expressed in MHz 
 *
 * @param baud_rate_KHz Baud rate expressed in KHz
 *
 * @return 
 *
 * @note  If other devices are connected on the I2C bus the baud rate would be changed by
 * the function. Among several calls from different device initializations, only the last one will be
 * reflected in the I2C module configuration. 
 *
 */
void DS1337_initialize (unsigned char crystal_frequency_MHz, unsigned int baud_rate_KHz);
#define initialize_DS1337 DS1337_initialize


/**
 * This function writes the seconds in the RTC.
 *
 * @param seconds Seconds to be written, as 2 digits BCD format (eg. 22 sec. is 0x22 or 0b00100010).
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Seconds are in BCD format. Decimal format won't work!
 */
signed char RTCC_set_seconds (unsigned char seconds);
#define set_seconds_RTCC RTCC_set_seconds


/**
 * This function reads the second from the RTC.
 *
 * @return Second that are read back, in BCD format (eg. 22 sec. is 0x22 or 0b00100010).
 *
 * @note Seconds are in BCD format.
 */
unsigned char RTCC_get_seconds (void);
#define get_seconds_RTCC RTCC_get_seconds


/**
 * This function writes the seconds in the RTC.
 *
 * @param minutes Minutes to be written, as 2 digits BCD format (eg. 22 min. is 0x22 or 0b00100010).
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Minutes are in BCD format. Decimal format won't work!
 */
signed char RTCC_set_minutes (unsigned char minutes);
#define set_minutes_RTCC RTCC_set_minutes

/**
 * This function reads the minutes from the RTC.
 *
 * @return Minutes that are read back, in BCD format (eg. 22 min. is 0x22 or 0b00100010).
 *
 * @note Seconds are in BCD format.
 */
unsigned char get_minutes_RTCC (void);
#define get_minutes_RTCC RTCC_get_minutes


/**
 * This function writes the hours in the RTC.
 *
 * @param hours Hours to be written, as 2 digits BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Hours are in BCD format. Decimal format won't work!
 */
signed char RTCC_set_hours (unsigned char hours);
#define set_hours_RTCC RTCC_set_hours


/**
 * This function reads the hours from the RTC.
 *
 * @return Hours that are read back, in BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @note Hours are in BCD format.
 */
unsigned char RTCC_get_hours (void);
#define get_hours_RTCC RTCC_get_hours


/**
 * This function reads the time from the RTC. The format is HH:MM.ss
 *
 * @return Time that is read back, as ASCII string terminated with /0
 *
 */
unsigned char* RTCC_get_time_seconds (void);
#define get_time_seconds_RTCC RTCC_get_time_seconds


/**
 * This function reads the time from the RTC. The format is HH:MM (without seconds)
 *
 * @return Time that is read back, as ASCII string terminated with /0
 *
 */
unsigned char* RTCC_get_time (void);
#define get_time_RTCC RTCC_get_time


/**
 * This function writes the days in the RTC.
 *
 * @param days Day to be written, as 2 digits BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Days are in BCD format. Decimal format won't work!
 */
signed char RTCC_set_days (unsigned char days);
#define set_days_RTCC RTCC_set_days


/**
 * This function reads the day from the RTC.
 *
 * @return Days that are read back, in BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @note Days are in BCD format.
 */
unsigned char RTCC_get_days (void);
#define get_days_RTCC RTCC_get_days



/**
 * This function writes the moths in the RTC.
 *
 * @param months Months to be written, as 2 digits BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Months are in BCD format. Decimal format won't work!
 */
signed char RTCC_set_months (unsigned char months);
#define set_months_RTCC RTCC_set_months


/**
 * This function reads the months from the RTC.
 *
 * @return Months that are read back, in BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @note Months are in BCD format.
 */
unsigned char RTCC_get_months (void);
#define get_months_RTCC RTCC_get_months


/**
 * This function writes the Year in the RTC.
 *
 * @param years Year to be written, as 2 digits BCD format (eg. 2012 is only 0x12 or 0b00010010).
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Years are in BCD format. Decimal format won't work!
 * @note      2012 is written as 12.
 */
signed char RTCC_set_years (unsigned char years);
#define set_years_RTCC RTCC_set_years


/**
 * This function reads the Year from the RTC.
 *
 * @return Year that is read back, in BCD format (eg. 12 is 0x12 or 0b00010010 represents 2012).
 *
 * @note Year are in BCD format. 2012 is read as 0x12.
 */
unsigned char RTCC_get_years (void);
#define get_years_RTCC RTCC_get_years



/**
 * This function reads the Date from the RTC. The Format DD/MM/YY
 *
 * @return Date that is read back, as ASCII string terminated with /0
 *
 */
unsigned char* RTCC_get_date (void);
#define get_date_RTCC RTCC_get_date


/**
 * This function writes the seconds for the alarm, in the RTC.
 *
 * @param seconds Seconds to be written, as 2 digits BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @param alarmEnable Enable the alarm or not (for matching) [ENABLE_ON , ENABLE_OFF]
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Seconds are in BCD format. Decimal format won't work!
 *
 * @note Use the constants ENABLE_ON and ENABLE_OFF
 */
signed char RTCC_set_seconds_alarm (unsigned char seconds, unsigned char alarm_enable);
#define set_seconds_alarm_RTCC RTCC_set_seconds_alarm

/**
 * This function writes the minutes for the alarm, in the RTC.
 *
 * @param minutes Minutes to be written, as 2 digits BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @param alarmEnable Enable the alarm or not (for matching) [ENABLE_ON , ENABLE_OFF]
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Minutes are in BCD format. Decimal format won't work!
 *
 * @note Use the constants ENABLE_ON and ENABLE_OFF
 */
signed char RTCC_set_minutes_alarm (unsigned char minutes, unsigned char alarm_enable);
#define set_minutes_alarm_RTCC RTCC_set_minutes_alarm


/**
 * This function writes the hours for the alarm, in the RTC.
 *
 * @param hours Hours to be written, as 2 digits BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @param alarmEnable Enable the alarm or not (for matching) [ENABLE_ON , ENABLE_OFF]
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Hours are in BCD format. Decimal format won't work!
 *
 * @note Use the constants ENABLE_ON and ENABLE_OFF
 */
signed char RTCC_set_hours_alarm (unsigned char hours, unsigned char alarm_enable);
#define set_hours_alarm_RTCC RTCC_set_hours_alarm

/**
 * This function writes the Days for the alarm, in the RTC.
 *
 * @param days Days to be written, as 2 digits BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @param alarmEnable Enable the alarm or not (for matching) [ENABLE_ON , ENABLE_OFF]
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Days are in BCD format. Decimal format won't work!
 *
 * @note Use the constants ENABLE_ON and ENABLE_OFF
 */
signed char RTCC_set_days_alarm (unsigned char days, unsigned char alarm_enable);
#define  set_days_alarm_RTCC RTCC_set_days_alarm


/**
 * This function enables the interrupt pin functionality.
 *
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 */
signed char RTCC_enable_alarm_interrupt (void);
#define enable_alarm_interrupt_RTCC RTCC_enable_alarm_interrupt


/**
 * This function disable the interrupt pin functionality.
 *
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 */
signed char RTCC_disable_alarm_interrupt (void);
#define disable_alarm_interrupt_RTCC RTCC_disable_alarm_interrupt


/**
 * This function checks if the Alarm id ON using polling method.
 *
 *
 * @return status   Alarm status 1: ON  0:OFF
 *
 * @note If the alarm is on, AF flag is automatically cleaned.
 */
unsigned char RTCC_is_alarm_ON (void);
#define is_alarm_ON_RTCC RTCC_is_alarm_ON


/**
 * This function increments the minutes inside the RTCC.
 *
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @note The function automatically checks the overflow and automatically increments the hours.
 */
signed char RTCC_increment_minutes (void);
#define increment_minutes_RTCC RTCC_increment_minutes


/**
 * This function increments the hours inside the RTCC.
 *
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @note The function automatically checks the overflow.
 */
signed char RTCC_increment_hours (void);
#define increment_hours_RTCC RTCC_increment_hours


/**
 * This function increments the Years inside the RTCC.
 *
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @note The function automatically checks the overflow.
 */
signed char RTCC_increment_years (void);
#define increment_years_RTCC RTCC_increment_years


/**
 * This function increments the Months inside the RTCC.
 *
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @note The function automatically checks the overflow.
 */
signed char RTCC_increment_months (void);
#define increment_months_RTCC RTCC_increment_months


/**
 * This function increments the Days inside the RTCC.
 *
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @note The function automatically checks the overflow.
 */

signed char RTCC_increment_days (void);
#define increment_days_RTCC RTCC_increment_days


#endif

