/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1

Created on Date : 08/09/2014
Last update     : 15/12/2014

CopyRight 2006-2014 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


********************************************************
PURPOSES
********************************************************

 This library allows the usage of the MCP4822 DAC.

*******************************************************************************/

#ifndef MCP4822_H
#define MCP4822_H

#define USE_OR_MASKS
#include <spi.h>

#ifdef __XC8
    #include <xc.h>
	
	#ifndef _PIC18
            #error The MCP4822 Library supports only PIC18 devices
    #endif
#endif

#ifdef __XC8
    #include <stdlib.h>
    #define MCP4822_PULSE_DELAY _delay(1)
#endif

#ifndef __XC8
    #include <delays.h>
    #define MCP4822_PULSE_DELAY Delay1TCY(1)
#endif
 

//**************************************************
// Pin initializations
//**************************************************

//#define MCP4822_SDO_BIT     7
#define MCP4822_SDO_BIT     5
#define MCP4822_SDO_TRIS    TRISC
#define MCP4822_SDO_PORT    LATC

//#define MCP4822_SCK_BIT     1
//#define MCP4822_SCK_TRIS    TRISB
//#define MCP4822_SCK_PORT    LATB
#define MCP4822_SCK_BIT     3
#define MCP4822_SCK_TRIS    TRISC
#define MCP4822_SCK_PORT    LATC

#define MCP4822_CS_BIT      5
#define MCP4822_CS_TRIS     TRISA
#define MCP4822_CS_PORT     LATA

#define MCP4822_LDAC_BIT    0
#define MCP4822_LDAC_TRIS   TRISE
#define MCP4822_LDAC_PORT   LATE


//**************************************************
// Constants
//**************************************************
#define MCP4822_DAC_A 0x00
#define MCP4822_DAC_B 0x80

#define MCP4822_GAIN_1 0x20
#define MCP4822_GAIN_2 0x00

#define MCP4822_OUT_DEFAULT_VALUE 0x00

#define MCP4822_SHUTDOWN_ON  0x00
#define MCP4822_SHUTDOWN_OFF 0x10

#define MCP4822_CLOCK_FOSC_64 SPI_FOSC_64
#define MCP4822_CLOCK_FOSC_16 SPI_FOSC_16
#define MCP4822_CLOCK_FOSC_4 SPI_FOSC_4
#define MCP4822_CLOCK_FOSC_TMR2 SPI_FOSC_TMR2


/**
 * This function initializes the MCP4822 and PIC signals. SDO, SS, SCK from the
 * PIC side are automatically set by this function.
 *
 * @param clock It sets the clock source for the SPI module. Use the following
 *              constants [MCP4822_CLOCK_FOSC_64, MCP4822_CLOCK_FOSC_16, MCP4822_CLOCK_FOSC_4, MCP4822_CLOCK_FOSC_TMR2]
 *
 * @return void
 *
 * @note By default the DAC outputs are set to high impedance.
 *
 * @warning You must properly set the Analog inputs which are muxed with the SPI lines.
 */
void MCP4822_initialize (unsigned char clock);
#define initialize_MCP4822 MCP4822_initialize


/**
 * This function sets the output value of the DAC. By Gain 2 the value is expressed in mV,
 * while by Gain 1 is 1/2 mV.
 *
 * @param value It sets the output value of the DAC [12 bits].
 *
 * @param channel It sets the channel to which the value refers to [MCP4822_DAC_A, MCP4822_DAC_B].
 *
 * @return void
 *
 * @note This function does not generate the LATCH pulse to load the DAC values.
 *       This it means that the pulse latch pulse must be generated separately.
 *       This option is used in case both DAC must be set before loading the
 *       output value.
 */
void MCP4822_set_value (unsigned int value, unsigned char channel);
#define set_value_MCP4822 MCP4822_set_value


/**
 * This function sets the output value of the DAC. By Gain 2 the value is expressed in mV,
 * while by Gain 1 is 1/2 mV.
 *
 * @param value It sets the output value of the DAC [12 bits].
 *
 * @param channel It sets the channel to which the value refers to [MCP4822_DAC_A, MCP4822_DAC_B].
 *
 * @return void
 *
 * @note This function does generate the LATCH pulse to load the DAC value right after
 *       the SPI writing.
 *
 */
void MCP4822_set_amplitude (unsigned int value, unsigned char channel);
#define set_amplitude_MCP4822 MCP4822_set_amplitude


/**
 * This function generates the LATCC pulse required to load the value to the DAC.
 * It is required only in case the MSP4822_set_value is used. The function
 * MSP4822_set_amplitude already include a call to this function.
 *
 * @param void
 *
 * @return void
 * 
 */
void MCP4822_latch_pulse (void);
#define latch_pulse_MCP4822 MCP4822_latch_pulse


/**
 * This function sets the DAC Options. The options which are set in the function,
 * take place after the next writing to the corresponding DAC Channel.
 *
 * @param gain It sets the output gain [MCP4822_GAIN_1, MCP4822_GAIN_2]
 *
 * @param power It sets the channel power mode [MCP4822_SHUTDOWN_ON, MCP4822_SHUTDOWN_OFF]
 *
 * @param channel It sets the channel to which the options refer to [MCP4822_DAC_A, MCP4822_DAC_B]
 *
 * @return void
 *
 * @note This function does NOT generate the LATCH pulse to load the DAC value right after
 *       the writing. The settings take place by the next amplitude writing. 
 *
 */
void MCP4822_set_options (unsigned char gain, unsigned char power, unsigned char channel);
#define set_options_MCP4822 MCP4822_set_options

#endif