/*******************************************************************************

Author : Mauro Laurenti
Version : 1.5
Created on Date : 09/03/2012
Last update     : 15/12/2014

CopyRight 2006-2014 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/
									
#ifdef __XC8
    #include <xc.h>
    #include <stdlib.h>

    #ifndef _PIC18
        #error The LCD_44780_I2C Library supports only PIC18 devices
    #endif
#endif

#include "LCD_44780_I2C.h"
#include "PCF8574.h"
#include "delay.h"

unsigned char data_buffer = 0;
unsigned char PCF8574_address = 0;

//************************************************************
//                  LCD_enable_pulse  Implementation
//************************************************************
void LCD_enable_pulse (void) {
	data_buffer = data_buffer | LCD_E;
	write_data_PCF8574 (PCF8574_address, data_buffer);
	delay_ms (1);
	data_buffer = data_buffer & ~LCD_E;
	write_data_PCF8574 (PCF8574_address, data_buffer);
}


//************************************************************
//                  LCD_send_command  Implementation
//************************************************************
void LCD_send_command (unsigned char data) {


	//clear the lower nible
	data_buffer = data_buffer & 0b11100000; 

	//write the data within the buffer
	data_buffer = data_buffer | data;

	write_data_PCF8574 (PCF8574_address, data_buffer);
	LCD_enable_pulse ();
}


//************************************************************
//                  LCD_home  Implementation
//************************************************************
void LCD_home (void) {
	
	LCD_send_command (0b00000000);
	LCD_send_command (0b00000010);
}


//************************************************************
//                  LCD_shift  Implementation
//************************************************************
void LCD_shift (unsigned char shift, unsigned char number_of_shift) {
	
	unsigned char i;
	
	for (i=0; i < number_of_shift; i++) {	
		LCD_send_command (0b00000001);
		LCD_send_command (0b00001000 | shift);
	}	
}


//************************************************************
//                  LCD_shift_cursor  Implementation
//************************************************************
void LCD_shift_cursor (unsigned char shift, unsigned char number_of_shift){
	
	unsigned char i;
	
	for (i=0; i < number_of_shift; i++) {
		LCD_send_command (0b00000001);
		LCD_send_command (0b00000000 | shift);
	} 		
}

//************************************************************
//                  LCD_goto_line  Implementation
//************************************************************
void LCD_goto_line (unsigned char line) {

switch(line) {

	case 1: LCD_send_command(0b00001000);
		LCD_send_command(0b00000000);
		break;

	case 2: LCD_send_command(0b00001100);
		LCD_send_command(0b00000000);
		break;

	case 3: LCD_send_command(0b00001001);
		LCD_send_command(0b00000100);
		break;

	case 4: LCD_send_command(0b00001101);
		LCD_send_command(0b00000100);
	}
}


//************************************************************
//                  LCD_goto_xy  Implementation
//************************************************************
void LCD_goto_xy (unsigned char x, unsigned char y){

    LCD_goto_line (y);
    LCD_shift_cursor (RIGHT, x-1);
}


//************************************************************
//                  LCD_write_char  Implementation
//************************************************************
void LCD_write_char (unsigned char value) {
	
	unsigned char preliminary_buffer;
	
	data_buffer = data_buffer | LCD_RS;
	write_data_PCF8574 (PCF8574_address, data_buffer);

	// Splitting of the first nibble
	preliminary_buffer = (value & 0xF0) >> 4;	
					
	LCD_send_command (preliminary_buffer);
		
	// Splitting of the second nibble
	preliminary_buffer = (value & 0x0F);
			
	LCD_send_command (preliminary_buffer);
		
	data_buffer = data_buffer & ~LCD_RS;
	write_data_PCF8574 (PCF8574_address, data_buffer);
}

//************************************************************
//                  LCD_write_message  Implementation
//************************************************************
#ifndef __XC8
void LCD_write_message (const rom unsigned char *buffer) {
#endif

#ifdef __XC8
void LCD_write_message (const unsigned char *buffer) {
#endif
	
	 // Write data to LCD up to null
    while(*buffer) {
	    
	    // Write character to LCD
	    LCD_write_char (*buffer);
	    // Increment buffer
	    buffer++;              
   }
}

//************************************************************
//                  LCD_write_string  Implementation
//************************************************************
void LCD_write_string (unsigned char *buffer) {
	
	// Write data to LCD up to null
	while(*buffer){
		
		// Write character to LCD
		LCD_write_char (*buffer);
		// Increment buffer
		buffer++;               
	}
}


//************************************************************
//                  LCD_write_integer  Implementation
//************************************************************
void LCD_write_integer (int value, unsigned char number_of_digits){
	
	// The array size is 5 plus end of string \0
	unsigned char convertedInt [6] = {0,0,0,0,0,0};
	
	// Index used to shift to the right the digit
	unsigned char index;
	
	// Integer is converted to string
        #ifndef __XC8
            itoa (value, (unsigned char*) convertedInt);
        #endif

        #ifdef __XC8
            itoa ((unsigned char*) convertedInt, value,10);
	#endif
	
	if (number_of_digits >0 ) {
		
		convertedInt[number_of_digits] = '\0';
		
		// Shift the digit to the right removing the empty one
		while (!(convertedInt[number_of_digits-1] <= '9' && convertedInt[number_of_digits-1] >= '0')) {
			
			for (index = number_of_digits-1; index > 0; index--){
				convertedInt[index] = convertedInt[index-1];
				convertedInt[index-1] = ' ';
			}							
		}	
	}

	LCD_write_string (convertedInt);
	
}	


//************************************************************
//                  LCD_clear  Implementation
//************************************************************
void LCD_clear (void){
	
	LCD_send_command (0b00000000);
	LCD_send_command (0b00000001);
}

//************************************************************
//                  cursor_LCD  Implementation
//************************************************************
void LCD_cursor (unsigned char active,unsigned char blinking) {
	
	LCD_send_command (0b00000000);
	LCD_send_command (0b00001100 | active | blinking);
}

//************************************************************
//                  LCD_backlight  Implementation
//************************************************************
void LCD_backlight (unsigned char active) {

	//Clear the LED bit
	data_buffer = data_buffer & ~LCD_TURN_ON_LED;

	//write the data within the buffer
	data_buffer = data_buffer | active;
	write_data_PCF8574 (PCF8574_address, data_buffer);
}

//************************************************************
//                  LCD_initialize  Implementation
//************************************************************
void  LCD_initialize (unsigned unsigned char quartz_frequency) {
	
	delay_set_quartz (quartz_frequency);

	initialize_PCF8574 (quartz_frequency, LCD_BUS_DATA_RATE);
	PCF8574_address = PCF8574_ADDRESS_H | (PCF8574_ADDRESS_L << 1);

	data_buffer = 0;
	write_data_PCF8574 (PCF8574_address, data_buffer);
	
	delay_ms (100);
	LCD_send_command (0b00000011);
	delay_ms (10);
	LCD_send_command (0b00000011);
	delay_ms (1);
	LCD_send_command (0b00000011);
		
	LCD_send_command (0b00000010);

	LCD_send_command (0b00000010);
	LCD_send_command (0b00001000);

	LCD_send_command (0b00000000);
	LCD_send_command (0b00001000);
        
        LCD_send_command (0b00000000);
	LCD_send_command (0b00000001);
        
        LCD_send_command (0b00000000);
	LCD_send_command (0b00000110);


	LCD_cursor (LCD_TURN_OFF_CURSOR, LCD_BLINKING_OFF);
	LCD_clear ();
}

